//This is a sample C program for Microsoft C/C++ which is to be linked 
//to PSIM via DLL.

//To compile it into DLL:

//   From the command window, run the command "cl /LD ms_user3.c"

//   From Miscrosoft Developer Studio:
//      - From the "File" menu, choose "New"/"Project Workspace", and
//        select "Dynamic-Link Library". Set the name as "ms_user3".
//      - Copy this sample file into the directory where the project
//        resides.
//      - From the "Insert" menu, choose "Files into Project", and select
//        "ms_user3.c".
//      - Choose active configuration to "Release". From the "Build" menu,
//        choose "Rebuild All".

//After the DLL file "ms_user3.dll" is generated, backup the default file
//into another file or directory, and copy your DLL file into the PSIM
//directory (and overwriting the existing file). You are then ready to run 
//PSIM with your DLL.


//This sample program calculates the rms value of a 60-Hz input in[0],
//which is sampled at a sampling rate of 2400 Hz. The rms output is
//stored in out[0], and the intermediate variable sum is stored in out[1].

//Activate (enable) the following line if the file is a C++ file (i.e. "ms_user3.cpp")
//extern "C"

//Do not change the following line. It's for DLL
__declspec(dllexport)

//You may change the variable names (say from "t" to "Time").
//But DO NOT change the function name, number of variables, variable type, and sequence.

// Variables:
//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)      

// The maximum length of the input and output array "in" and "out" is 20.

// Warning: Global variables above the function ms_user3 (t,delt,in,out)
//          are not allowed!!!

void ms_user3 (t, delt, in, out)

//Note that all the variables must be defined as "double"
double t, delt;
double *in, *out;

{
//Place your code here............begin

	static double nsum=0., sum=0., rms;
	double Ts, Tperiod;

// Fundamental frequency 60 Hz; sampling frequency 2400 Hz.
	Ts=1./2400.;
	Tperiod=1./60.;

	if (t >= nsum*Tperiod)
	{
		nsum=nsum+1.;
		rms = sqrt(sum*Ts/Tperiod);
		sum=0.;
	}

	out[0] = rms;
	sum=sum+in[0]*in[0];
	out[1]=sum;

//Place your code here............end
}
