//This is a sample C program for Microsoft C/C++ which is to be linked 
//to PSIM via DLL.

//To compile it into DLL:

//   From the command window, run the command "cl /LD ms_user4.c"

//   From Miscrosoft Developer Studio:
//      - From the "File" menu, choose "New"/"Project Workspace", and
//        select "Dynamic-Link Library". Set the name as "ms_user4".
//      - Copy this sample file into the directory where the project
//        resides.
//      - From the "Insert" menu, choose "Files into Project", and select
//        "ms_user4.c".
//      - Choose active configuration to "Release". From the "Build" menu,
//        choose "Rebuild All".

//After the DLL file "ms_user4.dll" is generated, backup the default file
//into another file or directory, and copy your DLL file into the PSIM
//directory (and overwriting the existing file). You are then ready to run 
//PSIM with your DLL.


//This sample program implement the control of the circuit "pfvi-dll.sch" 
//in a C routine. 
//	Input: in[0]=Vin; in[1]=iL; in[2]=Vo
//	Output: Vm=out[0]; iref=out[1]

//Activate (enable) the following line if the file is a C++ file (i.e. "ms_user4.cpp")
//extern "C"

//Do not change the following line. It's for DLL
__declspec(dllexport)

//You may change the variable names (say from "t" to "Time").
//But DO NOT change the function name, number of variables, variable type, and sequence.

// Variables:
//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)

// The maximum length of the input and output array "in" and "out" is 20.

// Warning: Global variables above the function ms_user4 (t,delt,in,out)
//          are not allowed!!!

void ms_user4 (t, delt, in, out)

//Note that all the variables must be defined as "double"
double t, delt;
double *in, *out;

{
//Place your code here............begin

	double Voref=10.5, Va, iref, iL, Vo, Vm;
	double errv, erri, Ts=33.33e-6;
	static double yv=0., yi=0., uv=0., ui=0.;
	
//Input
	Va=fabs(in[0]);
	iL=in[1];
	Vo=in[2];

//Outer Loop
	errv=Voref-Vo;
//	Trapezoidal Rule
	yv=yv+(33.33*errv+uv)*Ts/2.;

//	Backward Euler
//	yv=yv+33.33*errv*Ts;

	iref=(errv+yv)*Va;

//Inner Loop
	erri=iref-iL;
//	Trapezoidal Rule
	yi=yi+(3400.*erri+ui)*Ts/2.;

//	Backward Euler
//	yi=yi+3400.*erri*Ts;

	Vm=yi+0.3*erri;

//Store old values
	uv=33.33*errv;
	ui=3400.*erri;

//Output
	out[0]=Vm;
	out[1]=iref;


//Place your code here............end
}
