// This is a sample C program for Microsoft C/C++ which is to be linked 
// to PSIM via DLL.

// To compile it into DLL:

//   From the command window, run the command "cl /LD ms_user5.c"

//   From Miscrosoft Developer Studio:
//      - From the "File" menu, choose "New"/"Project Workspace", and
//        select "Dynamic-Link Library". Set the name as "ms_user5".
//      - Copy this sample file into the directory where the project
//        resides.
//      - From the "Insert" menu, choose "Files into Project", and select
//        "ms_user5.c".
//      - Choose active configuration to "Release". From the "Build" menu,
//        choose "Rebuild All".

// After the DLL file "ms_user5.dll" is generated, backup the default file
// into another file or directory, and copy your DLL file into the PSIM
// directory (and overwriting the existing file). You are then ready to run 
// PSIM with your DLL.


// This sample program implement the control of the circuit "pfvi-dlla.sch" 
// in a C routine. 
//	Input:  in[0]=Vin; in[1]=iL; in[2]=Vo
//	Output: gating=out[0]; iref=out[1]; npulse=out[2]

//Activate (enable) the following line if the file is a C++ file (i.e. "ms_user5.cpp")
//extern "C"

// Do not change the following line. It's for DLL
__declspec(dllexport)

// You may change the variable names (say from "t" to "Time").
// But DO NOT change the function name, number of variables, variable type, and sequence.

// Variables:
//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)

// The maximum length of the input and output array "in" and "out" is 20.

// Warning: Global variables above the function ms_user5 (t,delt,in,out)
//          are not allowed!!!

void ms_user5 (t, delt, in, out)

// Note that all the variables must be defined as "double"
double t, delt;
double *in, *out;

{
// Place your code here............begin

	double Voref=10.5, Vcarr=10., Va, iL, Vo, Vm;
	double errv, erri, Ts=33.33e-6;
	static double yv=0., yi=0., uv=0., ui=0., iref;
	static int count=0, Ncount, flagSample=1, npulse, npulse_next=0, gating;

// Calculate the no. of counts in one period
	Ncount=Ts/delt;

// Check if the counter reaches the end of the period. If yes,
// set the sampling flag to 1. 
	if (count == Ncount)
	{
		flagSample=1;
//      Reset the counter to 0.
		count=0;
//      Update the on-time pulse width
		npulse=npulse_next;
	}

// If the sampling flag is 1, sample the inputs and calculate the
// on-time pulse width npulse.
	if (flagSample == 1)
	{
//		Reset the sampling flag
		flagSample=0;

//		Sample the inputs
		Va=fabs(in[0]);
		iL=in[1];
		Vo=in[2];

//		Calculate the outer loop PI controller using Trapezoidal Rule
		errv=Voref-Vo;
		yv=yv+(33.33*errv+uv)*Ts/2.;

		iref=(errv+yv)*Va;

//		Calculate the inner loop PI controller
		erri=iref-iL;
		yi=yi+(3400.*erri+ui)*Ts/2.;

		Vm=yi+0.3*erri;

//		Calculate the on-time pulse width.
//      Note that this value is not used within this sampling period, but 
//      is used at the beginning of the next sampling period.
		npulse_next=Ncount * (Vm/Vcarr);

//		Store old values
		uv=33.33*errv;
		ui=3400.*erri;

	}

// Generate the switch gating signal
	if (count <= npulse )
	{gating=1;}
	else
	{gating=0;}

// Output
	out[0]=gating;
	out[1]=iref;
	out[2]=npulse;

// Note: Among the 3 output variables, "gating" is calculated at
// every time step, but "iref" and "npulse" are calculated only
// once in one sampling period. Both "gating" and "iref" can be
// defined as either static or non-static. 
	
// But if "iref" is defined as non-static, its value will be zero at 
// the points other than the sampling point. In order to see the 
// value at the sampling point, the print step Iprint in the simulation
// control in SIMCAD must be set to 1. Otherwise the sampling point 
// may be skipped, resulting in a waveform of all zero.


// The variable "npulse" must be declared as static as its
// value is used at every time step and must be retained.


// Increment the counter by 1
	count++;

// Place your code here............end
}
